import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { useState, useEffect, useCallback } from 'react';
import { get_prop_values, is_object, is_required, VALUE, ERROR } from './utils';
/**
 * useForm hooks to handle your validation in your forms
 *
 * @param {object} stateSchema stateSchema.
 * @param {object} stateValidatorSchema stateSchemaValidation to validate your forms in react.
 * @param {function} submitFormCallback function to be execute during form submission.
 */

function useForm() {
  var stateSchema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var stateValidatorSchema = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var submitFormCallback = arguments.length > 2 ? arguments[2] : undefined;

  var _useState = useState(stateSchema),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setStateSchema = _useState2[1];

  var _useState3 = useState(stateValidatorSchema),
      _useState4 = _slicedToArray(_useState3, 2),
      validatorSchema = _useState4[0],
      setValidatorSchema = _useState4[1];

  var _useState5 = useState(get_prop_values(state, VALUE)),
      _useState6 = _slicedToArray(_useState5, 2),
      values = _useState6[0],
      setValues = _useState6[1];

  var _useState7 = useState(get_prop_values(state, ERROR)),
      _useState8 = _slicedToArray(_useState7, 2),
      errors = _useState8[0],
      setErrors = _useState8[1];

  var _useState9 = useState(get_prop_values(state, false)),
      _useState10 = _slicedToArray(_useState9, 2),
      dirty = _useState10[0],
      setDirty = _useState10[1];

  var _useState11 = useState(true),
      _useState12 = _slicedToArray(_useState11, 2),
      disable = _useState12[0],
      setDisable = _useState12[1];

  var _useState13 = useState(false),
      _useState14 = _slicedToArray(_useState13, 2),
      isDirty = _useState14[0],
      setIsDirty = _useState14[1]; // Get a local copy of stateSchema


  useEffect(function () {
    setStateSchema(stateSchema);
    setInitialErrorState();
  }, []); // eslint-disable-line
  // Set a brand new field values and errors
  // If stateSchema changes

  useEffect(function () {
    var values = get_prop_values(state, VALUE);
    var errors = Object.keys(values).reduce(function (accu, curr) {
      accu[curr] = validateField(curr, values[curr]);
      return accu;
    }, {}); // Marked form as dirty if state was changed.

    setIsDirty(true);
    setValues(values);
    setErrors(errors);
  }, [state]); // eslint-disable-line
  // Run validation if validatorSchema was already set or has change...

  useEffect(function () {
    var errors = Object.keys(values).reduce(function (accu, curr) {
      accu[curr] = validateField(curr, values[curr]);
      return accu;
    }, {});
    setErrors(errors);
  }, [validatorSchema]); // eslint-disable-line
  // For every changed in our state this will be fired
  // To be able to disable the button

  useEffect(function () {
    if (isDirty) {
      setDisable(validateErrorState());
    }
  }, [errors, isDirty]); // eslint-disable-line
  // Set field value to specific field.

  var setFieldValue = function setFieldValue(_ref) {
    var name = _ref.name,
        value = _ref.value;
    setValues(function (prevState) {
      return _objectSpread(_objectSpread({}, prevState), {}, _defineProperty({}, name, value));
    });
    setDirty(function (prevState) {
      return _objectSpread(_objectSpread({}, prevState), {}, _defineProperty({}, name, true));
    });
  }; // Set to specific field.


  var setFieldError = function setFieldError(_ref2) {
    var name = _ref2.name,
        error = _ref2.error;
    setErrors(function (prevState) {
      return _objectSpread(_objectSpread({}, prevState), {}, _defineProperty({}, name, error));
    });
  }; // Function used to validate form fields


  var validateField = useCallback(function (name, value) {
    var fieldValidator = validatorSchema[name]; // Making sure that stateValidatorSchema name is same in
    // stateSchema

    if (!fieldValidator) {
      return;
    }

    var error = '';
    error = is_required(value, fieldValidator['required']);

    if (error) {
      return error;
    } // Bail out if field is not required and no value set.
    // To prevent proceeding to validator function


    if (!fieldValidator['required'] && !value) {
      return error;
    } // Run custom validator function


    if (!error && is_object(fieldValidator['validator'])) {
      // Test the function callback if the value is meet the criteria
      if (!fieldValidator['validator']['func'](value, values)) {
        error = fieldValidator['validator']['error'];
      }
    }

    if (!error && is_object(fieldValidator['compare'])) {
      var _fieldValidator$compa = fieldValidator.compare,
          to = _fieldValidator$compa.to,
          errorMessage = _fieldValidator$compa.error;

      if (to && errorMessage && values[to] !== '') {
        if (value !== values[to]) {
          error = errorMessage;
        } else {
          setFieldError({
            name: to,
            error: ''
          });
        }
      }
    }

    return error;
  }, [validatorSchema, values]); // Set Initial Error State
  // When hooks was first rendered...

  var setInitialErrorState = useCallback(function () {
    Object.keys(errors).map(function (name) {
      return setFieldError({
        name: name,
        error: validateField(name, values[name])
      });
    });
  }, [errors, values, validateField]); // Used to disable submit button if there's a value in errors
  // or the required field in state has no value.
  // Wrapped in useCallback to cached the function to avoid intensive memory leaked
  // in every re-render in component

  var validateErrorState = useCallback(function () {
    return Object.values(errors).some(function (error) {
      return error;
    });
  }, [errors]); // Use this callback function to safely submit the form
  // without any errors in state...

  var handleOnSubmit = useCallback(function (event) {
    event.preventDefault(); // Making sure that there's no error in the state
    // before calling the submit callback function

    if (!validateErrorState()) {
      submitFormCallback(values);
    }
  }, [validateErrorState, submitFormCallback, values]); // Event handler for handling changes in input.

  var handleOnChange = useCallback(function (event) {
    var name = event.target.name;
    var value = event.target.value;
    var error = validateField(name, value);
    setFieldValue({
      name: name,
      value: value
    });
    setFieldError({
      name: name,
      error: error
    });
  }, [validateField]);
  return {
    dirty: dirty,
    values: values,
    errors: errors,
    disable: disable,
    setStateSchema: setStateSchema,
    setValidatorSchema: setValidatorSchema,
    setFieldValue: setFieldValue,
    setFieldError: setFieldError,
    handleOnChange: handleOnChange,
    handleOnSubmit: handleOnSubmit,
    validateErrorState: validateErrorState
  };
}

export default useForm;